'use strict';

var _extends = require('@babel/runtime/helpers/extends');
var _slicedToArray = require('@babel/runtime/helpers/slicedToArray');
var React = require('react');
var state = require('@codemirror/state');
var view = require('@codemirror/view');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var _extends__default = /*#__PURE__*/_interopDefaultLegacy(_extends);
var _slicedToArray__default = /*#__PURE__*/_interopDefaultLegacy(_slicedToArray);
var React__default = /*#__PURE__*/_interopDefaultLegacy(React);

// credit: https://github.com/JakeGinnivan/merge-refs-hook

function setRefs(ref, value) {
  if (typeof ref === 'function') {
    ref(value);
  } else if (ref) {
    ref.current = value;
  }
}

function useMergeRefs(forwardedRef, localRef) {
  return React__default['default'].useCallback(value => {
    setRefs(forwardedRef, value);
    setRefs(localRef, value);
  }, [forwardedRef, localRef]);
}

var CodeMirror = /*#__PURE__*/React__default['default'].forwardRef((_ref, ref) => {
  var value = _ref.value,
      selection = _ref.selection,
      onEditorViewChange = _ref.onEditorViewChange,
      onEditorStateChange = _ref.onEditorStateChange,
      onUpdate = _ref.onUpdate,
      _ref$extensions = _ref.extensions,
      passedExtensions = _ref$extensions === void 0 ? [] : _ref$extensions,
      elementProps = _ref.elementProps;
  var innerRef = React__default['default'].useRef(null);
  var mergedRef = useMergeRefs(ref, innerRef);

  var _React$useState = React__default['default'].useState(null),
      _React$useState2 = _slicedToArray__default['default'](_React$useState, 2),
      editorView = _React$useState2[0],
      setEditorView = _React$useState2[1];

  React__default['default'].useEffect(() => {
    var currentEditor = innerRef.current;
    if (!currentEditor) return;
    var view$1 = new view.EditorView({
      parent: currentEditor
    });
    setEditorView(view$1);
    if (onEditorViewChange) onEditorViewChange(view$1);
    return () => view$1.destroy();
  }, [innerRef]);
  React__default['default'].useEffect(() => {
    if (!editorView) return;
    var extensions = [];
    if (onUpdate) extensions.push(view.EditorView.updateListener.of(onUpdate)); // keep current state, only change extensions

    var editorState = state.EditorState.create({
      doc: editorView.state.doc.toString(),
      selection: editorView.state.selection,
      extensions: [...extensions, ...passedExtensions]
    });
    editorView.setState(editorState);
    if (onEditorStateChange) onEditorStateChange(editorState);
  }, [passedExtensions, editorView]);
  React__default['default'].useEffect(() => {
    if (!editorView) return;
    var transaction = editorView.state.update({
      changes: {
        from: 0,
        to: editorView.state.doc.length,
        insert: value
      }
    });
    editorView.dispatch(transaction);
  }, [value, editorView]);
  React__default['default'].useEffect(() => {
    if (!editorView) return;
    var transaction = editorView.state.update({
      selection
    });
    editorView.dispatch(transaction);
  }, [selection, editorView]);
  return /*#__PURE__*/React__default['default'].createElement("div", _extends__default['default']({
    ref: mergedRef
  }, elementProps));
});
CodeMirror.displayName = 'CodeMirror';

module.exports = CodeMirror;
