import _extends from '@babel/runtime/helpers/esm/extends';
import _slicedToArray from '@babel/runtime/helpers/esm/slicedToArray';
import React from 'react';
import { EditorState } from '@codemirror/state';
import { EditorView } from '@codemirror/view';

// credit: https://github.com/JakeGinnivan/merge-refs-hook

function setRefs(ref, value) {
  if (typeof ref === 'function') {
    ref(value);
  } else if (ref) {
    ref.current = value;
  }
}

function useMergeRefs(forwardedRef, localRef) {
  return React.useCallback(value => {
    setRefs(forwardedRef, value);
    setRefs(localRef, value);
  }, [forwardedRef, localRef]);
}

var CodeMirror = /*#__PURE__*/React.forwardRef((_ref, ref) => {
  var value = _ref.value,
      selection = _ref.selection,
      onEditorViewChange = _ref.onEditorViewChange,
      onEditorStateChange = _ref.onEditorStateChange,
      onUpdate = _ref.onUpdate,
      _ref$extensions = _ref.extensions,
      passedExtensions = _ref$extensions === void 0 ? [] : _ref$extensions,
      elementProps = _ref.elementProps;
  var innerRef = React.useRef(null);
  var mergedRef = useMergeRefs(ref, innerRef);

  var _React$useState = React.useState(null),
      _React$useState2 = _slicedToArray(_React$useState, 2),
      editorView = _React$useState2[0],
      setEditorView = _React$useState2[1];

  React.useEffect(() => {
    var currentEditor = innerRef.current;
    if (!currentEditor) return;
    var view = new EditorView({
      parent: currentEditor
    });
    setEditorView(view);
    if (onEditorViewChange) onEditorViewChange(view);
    return () => view.destroy();
  }, [innerRef]);
  React.useEffect(() => {
    if (!editorView) return;
    var extensions = [];
    if (onUpdate) extensions.push(EditorView.updateListener.of(onUpdate)); // keep current state, only change extensions

    var editorState = EditorState.create({
      doc: editorView.state.doc.toString(),
      selection: editorView.state.selection,
      extensions: [...extensions, ...passedExtensions]
    });
    editorView.setState(editorState);
    if (onEditorStateChange) onEditorStateChange(editorState);
  }, [passedExtensions, editorView]);
  React.useEffect(() => {
    if (!editorView) return;
    var transaction = editorView.state.update({
      changes: {
        from: 0,
        to: editorView.state.doc.length,
        insert: value
      }
    });
    editorView.dispatch(transaction);
  }, [value, editorView]);
  React.useEffect(() => {
    if (!editorView) return;
    var transaction = editorView.state.update({
      selection
    });
    editorView.dispatch(transaction);
  }, [selection, editorView]);
  return /*#__PURE__*/React.createElement("div", _extends({
    ref: mergedRef
  }, elementProps));
});
CodeMirror.displayName = 'CodeMirror';

export default CodeMirror;
